/*                           -----IsoDoIt.C-----

Written by: F.N.Zihlman
compiler:   Turbo C V2.0

USAGE:  IsoDoIt(ioparms);

where:	* ioparms is the data structure containing the user-supplied I/O
	  parameters

function: 1) Open files
	  2) Set up graphics environment, viewports 1 & 2
	  3) Read input data
	  4) Set up data-dependent viewport parameters
	  5) Display data on a regional basis or as individual
	     velocity functions
	  6) Edit data
	  7) Close files
*/

#include <dos.h>
#include <stdio.h>
#include <alloc.h>
#include <string.h>
#include <graphics.h>
#include <conio.h>
#include <math.h>

#ifndef ISODEFSH
#define ISODEFSH 1
#include "isodefs.h"
#endif

#ifndef ISOPARMSH
#define ISOPARMSH 1
#include "isoparms.h"
#endif

#ifndef ISOTVH
#define ISOTVH 1
#include "isotv.h"
#endif

#ifndef ISOWINDSH
#define ISOWINDSH 1
#include "isowinds.h"
#endif



#define DBL   205
#define DUMP  0x5
#define ESC 0x1b
#define BIGMOVE 10

#define R_ARRO  77
#define L_ARRO  75
#define U_ARRO  72
#define D_ARRO  80
#define R_10    82
#define L_10    76
#define U_10    85
#define D_10    68
#define SHFTR   54
#define SHFTL   52
#define SHFTU   56
#define SHFTD   50

#define F1      59
#define F2      60
#define F3      61
#define F4      62
#define F5      63
#define F6      64
#define F7      65
#define F8      66
#define F9      67
#define F10     68

#define SHFTF3  86
#define SHFTF4  87

FILE *outfile;

extern int GraphDriver;
extern int GraphMode;

char *month[12]={ "January","February","March","April","May","June","July",
                  "August","September","October","November","December"};

/* ------------------------------------------------------------- */

void grsetup();      /* graphics setup                     */
void moveit();       /* move cursor about the screen       */
int  IsoParms();     /* input parameters                   */
int  IsoInput();     /* read CDP functions                 */
void IsoVpSet();     /* initial viewport parameters        */
void VP1Plot();      /* plot VP1 display                   */
void VP2Plot();      /* plot VP2 display                   */
void IsoEdit();      /* edit functions                     */
void IsoPlot();      /* calc & plot iso-lines              */
void HelpFile();     /* IsoDoIt help file                  */
void VeDoIt();	     /* edit individual velocity functions */

/* ============================================================= */

int IsoDoIt(ioparms)
PARAMETERS *ioparms;
{
  TVDATA datain[MAXCDPS];
  WINDOW1 VP1;
  WINDOW2 VP2;
  WINDOW3 VP3;

  FILE *prtfile;

  struct date pdate;
  struct time ptime;

  union  REGS regs;
  struct SREGS sregs;

  extern char *month[];
  char ttlstr[MAXTITLE];

  int defopen=FALSE,istat=TRUE;
  int VP2x,VP2y,crtmode,idx,fcolor,bcolor;
  int ch,done=FALSE,cirrad,i,i2;
  unsigned size;
  void * buffer;
  char str[MAXFLNM],DFlag;

  strcpy(ttlstr,VP1.title1.titl);
  strcpy(VP1.title2.titl," ");

  /* -----Open output file-------------------------------------------- */
  if((stricmp(ioparms->deffl,"none"))!=0)
    if((outfile=fopen(ioparms->deffl,"w"))==NULL)
    {
      printf("\n\nERROR OPENING OUTPUT FILE %s\n",ioparms->deffl);
      printf("Press any key to continue...");
      getch();
      istat=FALSE;
    }
    else
    {
      defopen=TRUE;
      strcpy(VP1.title2.titl,"Current output file: ");
      strcat(VP1.title2.titl,ioparms->deffl);
    }

  /*-----Process Loop-----------------------------------------*/
  if(istat==TRUE)
  {
    while(istat==TRUE)     /* PROCESSING LOPP */
    {
      istat=IsoParms(ioparms);
      if(istat==TRUE)
	istat=IsoInput(ioparms,datain);
      if(istat==TRUE)
      {
	grsetup();
	settextjustify(CENTER_TEXT,CENTER_TEXT);
	IsoVpSet(ioparms,datain,&VP1,&VP2,&VP3);
	fcolor=getcolor();
	bcolor=getbkcolor();

        /* -----plot view ports----- */
        VP1Plot(&VP1);
	VP2Plot(&VP2,ioparms,datain);

        /* -----set marker----- */
        cirrad=VP2.twth;
        size=imagesize(0,0,(2*cirrad+1),(2*cirrad+1));
        buffer=malloc(size);
        VP2x=50;  VP2y=50;
	getimage(VP2x-cirrad-1,VP2y-cirrad-1,VP2x+cirrad+1,
		 VP2y+cirrad+1,buffer);
	circle(VP2x,VP2y,cirrad);

        /* ===================================================== */
        /*            CURSOR MOVEMENT / EDIT                     */
        /* ======================================================*/

        done=FALSE;
        while(!done)
        {
	  settextstyle(SMALL_FONT,HORIZ_DIR,MAGFACTOR);
          ch=toupper(getch());
          switch(ch)
	  {
	    case R_ARRO:
	      DFlag='r';
	      moveit(VP2.xmax,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case L_ARRO:
	      DFlag='l';
	      moveit(VP2.xmin,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case D_ARRO:
	      DFlag='d';
	      moveit(VP2.ymax,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case U_ARRO:
	      DFlag='u';
	      moveit(VP2.ymin,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case R_10:
	    case (SHFTR):
	      DFlag='R';
	      moveit(VP2.xmax-BIGMOVE,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case (L_10):
	    case (SHFTL):
	      DFlag='L';
	      moveit(VP2.xmin+BIGMOVE,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case (U_10):
	    case (SHFTU):
	      DFlag='U';
	      moveit(VP2.ymin+BIGMOVE,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case (D_10):
	    case (SHFTD):
	      DFlag='D';
	      moveit(VP2.ymax-BIGMOVE,&VP2x,&VP2y,cirrad,buffer,DFlag);
	      break;

	    case F1:
	      IsoHelp(VP2.thgt);
	      break;

	    case F2:
	      setviewport(VP1.x1,VP1.y1,VP1.x2,VP1.y2,1);
	      settextjustify(CENTER_TEXT,CENTER_TEXT);
	      setcolor(bcolor);
	      outtextxy(VP1.title1.tx,VP1.title1.ty,ttlstr);
	      setcolor(fcolor);
	      outtextxy(VP1.title1.tx,VP1.title1.ty,VP1.title1.titl);
	      setviewport(VP2.x1,VP2.y1,VP2.x2,VP2.y2,1);
	      VP2Plot(&VP2,ioparms,datain);
	      break;

	    case SHFTF3:
	      clearviewport();
	    case F3:
		IsoPlot(&VP1,&VP2,ioparms,datain,ttlstr);
	      break;

	    case F4:
	      idx=-1;
	      for(i=0;i<ioparms->recsread;i++)
		if((VP2x>=VP2.xcoords[i]-cirrad)&&(VP2x<=VP2.xcoords[i]+cirrad))
		  idx=i;
	      if(idx>=0)
	      {
		VP2x=VP2.xcoords[idx];
		IsoEdit(&VP2x,&VP2y,&VP2,&datain[idx],
			ioparms->tmin,ioparms->tmax,buffer,&VP3);
	      }
	      break;

	    case F5:
	      if((strstr((getmodename(getgraphmode())),"CGA"))!=NULL)
	      {
		if((prtfile=fopen("prn","w"))!=NULL)
		{
		  getdate(&pdate);
		  gettime(&ptime);
		  for(i=0;i<80;i++)
		    fprintf(prtfile,"%c",DBL);
		  fprintf(prtfile,"\n");
		  sprintf(str,"%s %d, %d     %02d:%02d:%02d.%02d",
			month[pdate.da_mon-1],pdate.da_day,pdate.da_year,
			ptime.ti_hour,ptime.ti_min,ptime.ti_sec,
			ptime.ti_hund);
		  fprintf(prtfile,"%s\n\n",str);
		  sprintf(str,"DATA FILE:  %s",ioparms->srcfl);
		  fprintf(prtfile,"%s\n",str);
		  sprintf(str,"OUTPUT FILE: %s",ioparms->deffl);
		  fprintf(prtfile,"%s\n",str);
		  sprintf(str,"Velocity           min: %-6.0f     max: %-6.0f",
			ioparms->vmin,ioparms->vmax);
		  fprintf(prtfile,"%s\n",str);
		  sprintf(str,"Time               min: %-6.0f     max: %-6.0f",
		       ioparms->tmin,ioparms->tmax);
		  fprintf(prtfile,"%s\n",str);
		  for(i=0;i<80;i++)
		    fprintf(prtfile,"%c",DBL);
		  fprintf(prtfile,"\n");
		  fclose(prtfile);
		  int86x(DUMP,&regs,&regs,&sregs);
		}
	      }
	      break;

	    case F6:
	      if(defopen)
	      {
		if(ioparms->datype=='d')
		  IsoDef(datain,ioparms->recsread);
		else
		  IsoIso(datain,ioparms->recsread);
		setviewport(VP1.x1,VP1.y1,VP1.x2,VP1.y2,1);
		settextjustify(CENTER_TEXT,CENTER_TEXT);
		setcolor(bcolor);
		outtextxy(VP1.title2.tx,VP1.title2.ty,VP1.title2.titl);
		setcolor(fcolor);
		strcpy(VP1.title2.titl,"Current output file: ");
		strcat(VP1.title2.titl,ioparms->deffl);
		strcat(VP1.title2.titl,"...pts written: ");
		strcat(VP1.title2.titl,ioparms->cdp1);
		strcat(VP1.title2.titl," to ");
		strcat(VP1.title2.titl,ioparms->cdp2);
		outtextxy(VP1.title2.tx,VP1.title2.ty,VP1.title2.titl);
		setviewport(VP2.x1,VP2.y1,VP2.x2,VP2.y2,1);
	      }
	      break;

	    case F7:
	      idx=-1;
	      for(i=0;i<ioparms->recsread;i++)
		if((VP2x>=VP2.xcoords[i]-cirrad)&&(VP2x<=VP2.xcoords[i]+cirrad))
		  idx=i;
	      if(idx>=0)
		VeDoIt(&datain[idx],&VP3);
	      VP1Plot(&VP1);
	      VP2Plot(&VP2,ioparms,datain);
	      break;

            case 'Q':
	      done=TRUE;
	      free(buffer);
	      closegraph();
	      break;
          }
        }
      }
    }
    if(defopen)
      fclose(outfile);
    if((istat==FALSE)||(istat==QUITIT))
      istat=TRUE;
    return istat;
  }
}                         /* ----End IsoDoIt----- */
