/*                         -----IsoRead.c-----

Written by F.N.Zihlman
compiler: Turbo C V2.0

USAGE: istat=IsoRead(&datain)

where: * istat is the integer return status
       * datain is the data structure of type TVDATA to hold the input data

FUNCTION: Reads input file one cdp function at a time and:
          1) Stores cdp id
	  2) Stores time/velocity {depth} pairs into a 50X2 array
	  3) Determines the function's tmin,tmax,vmin{dmin},vmax{dmax}
	  4) Returns TRUE if successful
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#ifndef ISODEFSH
#define ISODEFSH 1
#include "isodefs.h"
#endif

#ifndef ISOTVH
#define ISOTVH 1
#include "isotv.h"
#endif

/* --------------------------------------------------------------- */

int IsoCdp(char iobuf[], char vid[]);		/* cdp number conversion */
int IsoTV(char iobuf[], float *t, float *v);	/* t/v number conversion */

/* =============================================================== */

int IsoRead(datastruc)
TVDATA *datastruc;
{
  char iobuf[IOBUFLEN],ctim[CLEN],cvel[CLEN],*ch;
  int istat=TRUE,eoread=FALSE,irec=0,velid=FALSE,i;
  extern FILE *infile;

  datastruc->tmin=99999;
  datastruc->tmax=0;
  datastruc->vmin=99999;
  datastruc->vmax=0;
  datastruc->numread=0;

  while(!eoread)
  {
    if((fgets(iobuf,sizeof(iobuf),infile))==NULL)
      eoread=TRUE;
    else
    {
      eoread=TRUE;
      for(i=0;i<strlen(iobuf);i++)
	if(!isspace(iobuf[i]))
	  eoread=FALSE;
    }

    if(!eoread)
      if(!velid)
      {
	istat=IsoCdp(iobuf,datastruc->id);
	if(istat==TRUE)
	  velid=TRUE;
	else
	  eoread=TRUE;
      }
      else
      {
	istat=IsoTV(iobuf,&datastruc->tv[irec][0],&datastruc->tv[irec][1]);
	if(istat==TRUE)
	{
          if(datastruc->tv[irec][0]<datastruc->tmin)
	    datastruc->tmin=datastruc->tv[irec][0];
	  if(datastruc->tv[irec][0]>datastruc->tmax)
	    datastruc->tmax=datastruc->tv[irec][0];
	  if(datastruc->tv[irec][1]<datastruc->vmin)
	    datastruc->vmin=datastruc->tv[irec][1];
	  if(datastruc->tv[irec][1]>datastruc->vmax)
	    datastruc->vmax=datastruc->tv[irec][1];

          irec++;

          datastruc->numread=irec;
          if(irec>=MAXROWS)
          {
            eoread=TRUE;
            while(((fgets(iobuf,sizeof(iobuf),infile))!=NULL)&&(iobuf[0]!='\r')&&
	         (iobuf[0]!='\n'));
          }
	}
      }
  }
  if(datastruc->numread<=0)
    istat=FALSE;
  return(istat);
}                          /* -----End IsoRead----- */