/*                        -----VeDoIt.C-----

Written by: F.N.Zihlman
compiler:   Turbo C V2.0

USAGE:  VeDoIt(&datain,&VP3);

where:	* datain is the data structure containing the input data
	* VP3 is the data structure defining window 3

function: 1) Set up graphics environment, viewports 1 & 2
	  2) Display data as an individual velocity function
	  3) Edit data
*/

#include <dos.h>
#include <stdio.h>
#include <alloc.h>
#include <graphics.h>
#include <conio.h>
#include <math.h>

#ifndef ISODEFSH
#define ISODEFSH 1
#include "isodefs.h"
#endif

#ifndef ISOWINDSH
#define ISOWINDSH 1
#include "vedwinds.h"
#endif

#ifndef ISOTVH
#define ISOTVH 1
#include "isotv.h"
#endif

typedef struct {
        struct viewporttype VPort;
	int    x1,y1,x2,y2;
	int    thgt,twidth;
	int    tx,ty,vx,vy;
	float  tval,vval;
	} WINDOW3;

#define DBL   205
#define DUMP  0x5
#define ESC 0x1b
#define BIGMOVE 10

#define R_ARRO  77
#define L_ARRO  75
#define U_ARRO  72
#define D_ARRO  80
#define R_10    82
#define L_10    76
#define U_10    85
#define D_10    68
#define SHFTR   54
#define SHFTL   52
#define SHFTU   56
#define SHFTD   50

#define F1      59
#define F2      60
#define F3      61
#define F4      62
#define F5      63
#define F6      64
#define F7      65
#define F8      66
#define F9      67
#define F10     68

#define SHFTF2  85
#define SHFTF3  86

extern int GraphDriver;
extern int GraphMode;

/* ------------------------------------------------------------- */

void moveit();	      /* move cursor about the screen        */
void VedVpSet();      /* Initial viewport parameters         */
void Ved1Plot();      /* Plot Ved1 display                   */
void Ved2Plot();      /* Plot Ved2 display                   */
void DelPt();	      /* Delete a time/velocity pair         */
void AddPt();         /* Add a time/velocity pair            */
void TVDisp();	      /* Display time/velocity in window 3   */
float IsoNewV();      /* Calc a new velocity                 */
float IsoNewT();      /* Calc a new time                     */
void VedHelp();       /* help file                           */

/* ============================================================= */

void VeDoIt(datain,VP3)
TVDATA *datain;
WINDOW3 *VP3;
{
  VEDPORT1 Ved1;
  VEDPORT2 Ved2;

  struct date pdate;
  struct time ptime;

  union  REGS regs;
  struct SREGS sregs;

  extern char *month[];
  char ttlstr[MAXTITLE];

  int Ved2x,Ved2y,crtmode,idx,fcolor,bcolor;
  int ch,done=FALSE,cirrad,i,i2;
  unsigned size;
  void * buffer;
  char str[MAXFLNM],DFlag,stuff[MAXFLNM];

  FILE *prtfile;

  strcpy(ttlstr,Ved1.title1.titl);
  strcpy(Ved1.title2.titl," ");

  /* -----set up VED viewports----- */
  VedVpSet(datain,&Ved1,&Ved2);

  /* -----plot view ports----- */
  Ved1Plot(&Ved1);
  Ved2Plot(&Ved1,&Ved2,datain);

  /* -----set marker----- */

  cirrad=Ved2.twth;
  size=imagesize(0,0,(2*cirrad+1),(2*cirrad+1));
  buffer=malloc(size);
  Ved2x=50;  Ved2y=50;
  getimage(Ved2x-cirrad-1,Ved2y-cirrad-1,Ved2x+cirrad+1,
	   Ved2y+cirrad+1,buffer);
  circle(Ved2x,Ved2y,cirrad);

  /* ===================================================== */
  /*            CURSOR MOVEMENT / EDIT                     */
  /* ======================================================*/

  done=FALSE;
  while(!done)
  {
    settextstyle(SMALL_FONT,HORIZ_DIR,MAGFACTOR);
    ch=toupper(getch());
    switch(ch)
    {
      case R_ARRO:
	DFlag='r';
	moveit(Ved2.xmax,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
	VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case L_ARRO:
	DFlag='l';
	moveit(Ved2.xmin,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case D_ARRO:
	DFlag='d';
	moveit(Ved2.ymax,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case U_ARRO:
	DFlag='u';
	moveit(Ved2.ymin,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case R_10:
      case (SHFTR):
	DFlag='R';
	moveit(Ved2.xmax-BIGMOVE,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case (L_10):
      case (SHFTL):
	DFlag='L';
	moveit(Ved2.xmin+BIGMOVE,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case (U_10):
      case (SHFTU):
	DFlag='U';
	moveit(Ved2.ymin+BIGMOVE,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case (D_10):
      case (SHFTD):
	DFlag='D';
	moveit(Ved2.ymax-BIGMOVE,&Ved2x,&Ved2y,cirrad,buffer,DFlag);
        VP3->tval=IsoNewT(Ved2y,Ved2.ymin,Ved2.ymax,datain->tmin,datain->tmax);
	VP3->vval=IsoNewT(Ved2x,Ved2.xmin,Ved2.xmax,datain->vmin,datain->vmax);
	TVDisp(VP3);
	setviewport(Ved2.x1,Ved2.y1,Ved2.x2,Ved2.y2,1);
	break;

      case F1:
	VedHelp(Ved2.thgt);
	break;

      case F2:
	Ved2Plot(&Ved1,&Ved2,datain);
	getimage(Ved2x-cirrad-1,Ved2y-cirrad-1,Ved2x+cirrad+1,
		 Ved2y+cirrad+1,buffer);
	circle(Ved2x,Ved2y,cirrad);
	break;

      case F3:
	if(datain->numread>0){
	  DelPt(datain,Ved2x,Ved2y,Ved2.xmin,Ved2.xmax,Ved2.ymin,
		Ved2.ymax,cirrad);
	  Ved2Plot(&Ved1,&Ved2,datain);
	  getimage(Ved2x-cirrad-1,Ved2y-cirrad-1,Ved2x+cirrad+1,
		   Ved2y+cirrad+1,buffer);
	  circle(Ved2x,Ved2y,cirrad);
	}
	break;

      case F4:
	if(datain->numread<49){
	  AddPt(datain,Ved2x,Ved2y,Ved2.xmin,Ved2.xmax,Ved2.ymin,
		Ved2.ymax,cirrad);
	  Ved2Plot(&Ved1,&Ved2,datain);
	  getimage(Ved2x-cirrad-1,Ved2y-cirrad-1,Ved2x+cirrad+1,
		   Ved2y+cirrad+1,buffer);
	  circle(Ved2x,Ved2y,cirrad);
	}
	break;

      case F5:
	if((prtfile=fopen("prn","w"))!=NULL)
	{
	  getdate(&pdate);
	  gettime(&ptime);
	  for(i=0;i<80;i++)
	    fprintf(prtfile,"%c",DBL);
	  fprintf(prtfile,"\n");
	  sprintf(str,"%s %d, %d     %02d:%02d:%02d.%02d",
		  month[pdate.da_mon-1],pdate.da_day,pdate.da_year,
		  ptime.ti_hour,ptime.ti_min,ptime.ti_sec,
		  ptime.ti_hund);
	  fprintf(prtfile,"%s\n\n",str);
	  sprintf(str,"Velocity           min: %-6.0f     max: %-6.0f",
		  datain->vmin,datain->vmax);
	  fprintf(prtfile,"%s\n",str);
	  sprintf(str,"Time               min: %-6.0f     max: %-6.0f",
		       datain->tmin,datain->tmax);
	  fprintf(prtfile,"%s\n",str);
	  for(i=0;i<80;i++)
	    fprintf(prtfile,"%c",DBL);
	  fprintf(prtfile,"\n");
	  fclose(prtfile);
	  int86x(DUMP,&regs,&regs,&sregs);
	}
	break;

	case 'Q':
	  done=TRUE;
	  free(buffer);
	  break;
    }
  }

}                         /* ----End VeDoIt----- */
