/*
			 ----- IsoVpSet.c -----

Written by F.N. Zihlman
Compiler: Turbo C V2.0

USAGE: IsoVpSet(ioparms,datain,VP1,VP2,VP3);

where: * ioparms is the data structure containing the user-supplied i/o
	 parameters
       * datain is the array of data structures for the velocity functions
       * VP1, VP2, VP3 are the data structures which define the three
	 graphics windows of the displays.

function: initialization of the three graphics windows, or "view ports",
	  used
*/

#include <stdio.h>
#include <dos.h>
#include <graphics.h>
#include <math.h>

#ifndef ISODEFSH
#define ISODEFSH 1
#include "isodefs.h"
#endif

#ifndef ISOPARMSH
#define ISOPARMSH 1
#include "isoparms.h"
#endif

#ifndef ISOTVH
#define ISOTVH 1
#include "isotv.h"
#endif

#ifndef ISOWINDSH
#define ISOWINDSH 1
#include "isowinds.h"
#endif

#define XORIGIN 7
#define YORIGIN 5

extern int GraphDriver,GraphMode;

/* ============================================================== */

void IsoVpSet(ioparms,datain,VP1,VP2,VP3)
PARAMETERS *ioparms;
TVDATA     *datain;
WINDOW1    *VP1;
WINDOW2    *VP2;
WINDOW3    *VP3;
{

  struct linesettingstype VP1lines;
  struct date pdate;
  struct time ptime;
  char dstr[30],tstr[30];
  float deltax,deltay,deltacdp,cdp1;
  int i;



  setviewport(VP1->x1,VP1->y1,VP1->x2,VP1->y2,1);

  settextstyle(SMALL_FONT,HORIZ_DIR,MAGFACTOR);
  settextjustify(CENTER_TEXT,CENTER_TEXT);

  getviewsettings(&VP1->VPort);
  getlinesettings(&VP1lines);

  VP1->x1=0;                                       /* VP1 GLOBAL VALUES */
  VP1->y1=0;
  VP1->x2=getmaxx();
  VP1->y2=getmaxy();
  VP1->thgt=textheight("Z");
  VP1->twidth=textwidth("Z");


  VP1->boxx1=XORIGIN*VP1->twidth-VP1lines.thickness;   /* VP1 BOX VALUES */
  VP1->boxy1=YORIGIN*VP1->thgt-VP1lines.thickness;
  VP1->boxx2=(getmaxx()-(2*VP1->twidth))+VP1lines.thickness;
  VP1->boxy2=(getmaxy()-(2*VP1->thgt))+VP1lines.thickness;

  strcpy(VP1->xaxis.label,"Location");                 /* AXES LABELS */
  VP1->xaxis.lx=(VP1->boxx2-VP1->boxx1)/2+VP1->boxx1;
  VP1->xaxis.ly=VP1->boxy1-(2*VP1->twidth);
  strcpy(VP1->yaxis.label,"Time (ms)");
  VP1->yaxis.lx=VP1->boxx1-(2*VP1->thgt);
  VP1->yaxis.ly=(VP1->boxy2-VP1->boxy1)/2+VP1->boxy1;

  strcpy(VP1->xaxis.pt1,ioparms->cdp1);                /* BOX      */
  VP1->xaxis.pt1x=(XORIGIN*VP1->twidth)+VP1->twidth+2; /* LOCATION */
  VP1->xaxis.pt1y=VP1->xaxis.ly;                       /* LABELS   */
  strcpy(VP1->xaxis.pt2,ioparms->cdp2);
  VP1->xaxis.pt2x=VP1->boxx2-VP1->twidth-2;
  VP1->xaxis.pt2y=VP1->xaxis.pt1y;

  if(ioparms->tstart<0.)       /* BOX TIME LABELS */
    sprintf(VP1->yaxis.pt1,"%.0f",ioparms->tmin);
  else
    sprintf(VP1->yaxis.pt1,"%.0f",ioparms->tstart);
  VP1->yaxis.pt1x=(XORIGIN-(strlen(VP1->yaxis.pt1))-1)*VP1->twidth;
  VP1->yaxis.pt1y=(VP1->boxy1+VP1lines.thickness)+VP1->thgt;

  if(ioparms->tend<0.)
    sprintf(VP1->yaxis.pt2,"%.0f",ioparms->tmax);
  else
    sprintf(VP1->yaxis.pt2,"%.0f",ioparms->tend);
  VP1->yaxis.pt2x=(XORIGIN-(strlen(VP1->yaxis.pt2))-1)*VP1->twidth;
  VP1->yaxis.pt2y=(VP1->boxy2-VP1lines.thickness)-VP1->thgt;

  VP1->xaxis.tic1x=VP1->xaxis.pt1x;                    /* BOX TICK MARKS */
  VP1->xaxis.tic1y=VP1->xaxis.pt1y+VP1->thgt;
  VP1->xaxis.tic2x=VP1->xaxis.pt2x;
  VP1->xaxis.tic2y=VP1->xaxis.tic1y;

  VP1->yaxis.tic1x=6*VP1->twidth;
  VP1->yaxis.tic1y=VP1->yaxis.pt1y;
  VP1->yaxis.tic2x=VP1->yaxis.tic1x;
  VP1->yaxis.tic2y=VP1->yaxis.pt2y;

  getdate(&pdate);                                 /* VP1 DISPLAY TITLES */
  gettime(&ptime);
  sprintf(dstr,"%d %d %d",pdate.da_mon,pdate.da_day,pdate.da_year);
  sprintf(tstr,"%02d:%02d:%02d.%02d",ptime.ti_hour,ptime.ti_min,ptime.ti_sec,ptime.ti_hund);
  strcat((strcat((strcpy(VP1->title1.titl,dstr)),
	" * Interactive Velocity Edit V0.2 * ")),tstr);
  VP1->title1.tx=VP1->xaxis.lx;
  VP1->title1.ty=VP1->thgt/2;
  VP1->title2.tx=VP1->title1.tx;
  VP1->title2.ty=VP1->y2-VP1->thgt;

  VP2->x1=VP1->boxx1+VP1lines.thickness;           /* VP2 SETUP         */
  VP2->y1=VP1->boxy1+VP1lines.thickness;
  VP2->x2=VP1->boxx2-VP1lines.thickness;
  VP2->y2=VP1->boxy2-VP1lines.thickness;

					 /* VP2 PLOTTING MIN/MAX VALUES */
  VP2->xmin=VP1->xaxis.tic1x-VP2->x1-1;
  VP2->ymin=VP1->yaxis.tic1y-VP2->y1+1;
  VP2->xmax=VP1->xaxis.tic2x-VP2->x1-1;
  VP2->ymax=VP1->yaxis.tic2y-VP2->y1+1;
  VP2->thgt=VP1->thgt;
  VP2->twth=VP1->twidth;

  deltax=((float)VP2->xmax)-((float)VP2->xmin); /* VP2 DATA PTS X COORDS */
  cdp1=atof(ioparms->cdp1);
  deltacdp=(atof(ioparms->cdp2))-cdp1;
  for(i=0;i<ioparms->recsread;i++)
  {
    VP2->xcoords[i]=(int)
    ((((atof(datain->id))-cdp1)*deltax/deltacdp)+VP2->xmin);
    datain++;
  }

  VP3->thgt=VP1->thgt;			/* VP3 SETUP */
  VP3->twidth=VP1->twidth;
  VP3->x1=VP1->x2-(20*VP1->twidth);
  VP3->y1=VP1->y1+VP1->thgt/2;
  VP3->x2=VP1->x2-(5*VP1->twidth);
  VP3->y2=VP1->y1+(2*VP1->thgt)+VP1->thgt/2;
  VP3->tx=2;
  VP3->ty=4;
  VP3->vx=VP3->tx;
  VP3->vy=12;

}